# https://afni.nimh.nih.gov/pub/dist/doc/htmldoc/background_install/install_instructs/steps_macOS_12_Silicon.html


export INSTALL_DIR=/Users/xuagu37/software/sse/manual/AFNI/24.11
mkdir -p ${INSTALL_DIR}

cd ${INSTALL_DIR}
curl -O https://raw.githubusercontent.com/afni/afni/master/src/other_builds/OS_notes.macos_12_ARM_a_admin_pt1.zsh
curl -O https://raw.githubusercontent.com/afni/afni/master/src/other_builds/OS_notes.macos_12_ARM_a_admin_pt2.zsh
curl -O https://raw.githubusercontent.com/afni/afni/master/src/other_builds/OS_notes.macos_12_ARM_b_user.tcsh


brew analytics off
brew install python netpbm cmake gfortran

# installing XQuartz (needs sudo)
brew install --cask xquartz

# install build packages
brew install libpng jpeg expat freetype fontconfig openmotif  \
             libomp gsl glib pkg-config gcc@13 libiconv autoconf \
             libxt mesa mesa-glu libxpm

# ----------------------------------------------------------------------
# put python in path and install matplotlib via pip (multiple shells)

export PATH=${PATH}:/opt/homebrew/opt/python/libexec/bin
echo 'export PATH=${PATH}:/opt/homebrew/opt/python/libexec/bin' >> ~/.bashrc

# install R - currently 4.3.1
curl -O https://cran.r-project.org/bin/macosx/big-sur-arm64/base/R-4.3.1-arm64.pkg
sudo installer -pkg R-4.3.1-arm64.pkg -target /
# and reboot : sudo reboot


# set up terminal
defaults write org.macosforge.xquartz.X11 wm_ffm -bool true
defaults write org.x.X11 wm_ffm -bool true
defaults write com.apple.Terminal FocusFollowsMouse -string YES



# install initial AFNI package (no binaries), to be able to run AFNI scripts
# install under initial abin (relevant if we -do_extras)

curl -O https://afni.nimh.nih.gov/pub/dist/bin/misc/@update.afni.binaries
tcsh @update.afni.binaries -no_recur -package anyos_text_atlas \
        -bindir  ${INSTALL_DIR}/abin

${INSTALL_DIR}/abin/init_user_dotfiles.py -shell_list bash zsh tcsh \
        -do_updates path apsearch -dir_bin ${INSTALL_DIR}/abin

# put AFNI in PATH
source ~/.bashrc

export package=macos_13_ARM_clang
build_afni.py -build_root ${INSTALL_DIR}/afni_build -package $package



export rver=`R --version | head -n 1 | cut -d ' ' -f 3`
export R_LIBS=${INSTALL_DIR}/sw/R-$rver

echo "export R_LIBS=$R_LIBS" >> ~/.bashrc
mkdir -p $R_LIBS

rPkgsInstall -pkgs ALL 2>&1 | tee out.rPkgsInstall.txt

# ----------------------------------------------------------------------
# verify that Xvfb is in the PATH
which Xvfb >& /dev/null
if ( $status ) then
   # add /opt/X11/bin to PATH
   echo 'export PATH=$PATH:/opt/X11/bin' >> ~/.bashrc
endif

# ----------------------------------------------------------------------
# misc, for ASC whining

cp ${INSTALL_DIR}/abin/AFNI.afnirc ~/.afnirc
suma -update_env
apsearch -update_all_afni_help

# ----------------------------------------------------------------------
# check the state of things

afni_system_check.py -check_all 2>&1 | tee out.ASC.txt

# ----------------------------------------------------------------------
# suggest what users might do to use AFNI right now
echo ""
echo "to use AFNI now (in zsh), start with:"
echo '   source ~/.bashrc'
echo ""
