#!/bin/bash
set -e

# CUDA 10.2
# cuDNN 8.2.2
# NCCL 2.11.4

# Get the release blob + patches
wget https://developer.download.nvidia.com/compute/cuda/10.2/Prod/local_installers/cuda_10.2.89_440.33.01_linux.run
wget https://developer.download.nvidia.com/compute/cuda/10.2/Prod/patches/1/cuda_10.2.1_linux.run
wget https://developer.download.nvidia.com/compute/cuda/10.2/Prod/patches/2/cuda_10.2.2_linux.run
md5sum *.run

export CUDA_HOME="/software/sse/manual/CUDA/10.2.89_440.33.01"

# This should match against:
# 455217d400c7627214d36ffafd359414
# 3d816c583759efa8567a92c1a2af90db
# 0ead65db6128c94286a1cdd1be0e12c4
# If OK, to install issue commands
sh ./cuda_10.2.89_440.33.01_linux.run --toolkit --silent --installpath=${CUDA_HOME}
rm /tmp/cuda-installer.log
sh ./cuda_10.2.1_linux.run --toolkit --silent --installpath=${CUDA_HOME}
rm /tmp/cuda-installer.log
sh ./cuda_10.2.2_linux.run --toolkit --silent --installpath=${CUDA_HOME}
rm /tmp/cuda-installer.log
# The installer leaves a log file in /tmp which blocks any other user's attempt
# at installing a personal CUDA installation. Remove that log file between install steps.

# Now install cuDNN. Get the tgz file and untar it. The dist tgz file must be
# downloaded separately while logged in at NVIDIA website.
tar xf cudnn-10.2-linux-x64-v8.2.2.26.tgz

# Install with
cp cuda/include/cudnn*.h ${CUDA_HOME}/include/
cp -P cuda/lib64/libcudnn* ${CUDA_HOME}/lib64/
chmod a+r ${CUDA_HOME}/include/cudnn*.h ${CUDA_HOME}/lib64/libcudnn*

# Install the NCCL communication libs and headers.The dist tarball file must be
# downloaded separately while logged in at NVIDIA website.
tar xf nccl_2.11.4-1+cuda10.2_x86_64.txz && cd nccl_2.11.4-1+cuda10.2_x86_64
cp -r lib/*${CUDA_HOME}/lib64/
cp -r include/nccl* ${CUDA_HOME}/include/

# Should there be a need to remove the NCCL installation, this can be done with
# find ${CUDA_HOME}/ -name "*nccl*" -exec rm '{}' \;

