#!/bin/bash
set -e

# CUDA 11.7.1
# cuDNN 8.5.0.96
# NCCL 2.14.3

module purge --force #Gets us the system compiler

MODULE_NAME=CUDA
MODULE_VERSION=11.7.1
MODULE_INSTALL_DIR=/software/sse/manual/${MODULE_NAME}/${MODULE_VERSION}
mkdir -p ${MODULE_INSTALL_DIR}
rm -rf ${MODULE_INSTALL_DIR}/*
cd ${MODULE_INSTALL_DIR}

wget https://developer.download.nvidia.com/compute/cuda/11.7.1/local_installers/cuda_11.7.1_515.65.01_linux.run
sh cuda_11.7.1_515.65.01_linux.run --toolkit --silent --installpath=${MODULE_INSTALL_DIR}
rm /tmp/cuda-installer.log


# Get the cuDNN toolkit separately and transfer to your location. You must log in
# on your NVIDIA account at nvidia.com to get access to the download. Now install.

wget -O cudnn-linux-x86_64-8.5.0.96_cuda11-archive.tar.xz "https://liuonline-my.sharepoint.com/:u:/g/personal/xuagu37_liu_se/EamBNODY7UJGv8_BW0qxYuoBye-A6NY76Rb-tH7EjOiBwA?e=Ic0wN6&download=1"

tar xf cudnn-linux-x86_64-8.5.0.96_cuda11-archive.tar.xz
cp cudnn-linux-x86_64-8.5.0.96_cuda11-archive/include/cudnn*.h ${MODULE_INSTALL_DIR}/include/
cp -P cudnn-linux-x86_64-8.5.0.96_cuda11-archive/lib/libcudnn* ${MODULE_INSTALL_DIR}/lib64/
chmod a+r ${MODULE_INSTALL_DIR}/include/cudnn*.h ${MODULE_INSTALL_DIR}/lib64/libcudnn*

# Install the NCCL communication libs and headers.The dist tarball file must be
# downloaded separately at https://developer.download.nvidia.com/compute/redist/nccl/
wget -O nccl_2.14.3-1+cuda11.7_x86_64.txz "https://liuonline-my.sharepoint.com/:u:/g/personal/xuagu37_liu_se/ER6A_C6DdlRNusxKUQ7aWaMBnUdyBu0sia9KAVPzDcFGBQ?e=uTIh2Z&download=1"
tar xf nccl_2.14.3-1+cuda11.7_x86_64.txz 
cp -r nccl_2.14.3-1+cuda11.7_x86_64/lib/* ${MODULE_INSTALL_DIR}/lib64/
cp -r nccl_2.14.3-1+cuda11.7_x86_64/include/nccl* ${MODULE_INSTALL_DIR}/include/

# Should there be a need to remove the NCCL installation, this can be done with
# find ${MODULE_INSTALL_DIR}/ -name "*nccl*" -exec rm '{}' \;

rm -rf cuda_11.7.1_515.65.01_linux.run cudnn-linux-x86_64-8.5.0.96_cuda11-archive* nccl_2.14.3-1+cuda11.7_x86_64*

mkdir -p /software/sse/modules/${MODULE_NAME}
BMD=/proj/nsc_testing/xuan/berzelius-module-builds
cp $BMD/${MODULE_NAME}/${MODULE_NAME}-${MODULE_VERSION}.lua /software/sse/modules/${MODULE_NAME}/.${MODULE_VERSION}.lua