#!/bin/bash
set -e

# CUDA 12.1.1
# cuDNN 8.9.4
# NCCL 2.18.3

module rm mpprun/4.1.5

export INSTALL_DIR=/software/sse/manual/CUDA/12.1.1_530.30.02
mkdir -p ${INSTALL_DIR}
cd ${INSTALL_DIR}

export CUDA_HOME=${INSTALL_DIR}
wget https://developer.download.nvidia.com/compute/cuda/12.1.1/local_installers/cuda_12.1.1_530.30.02_linux.run

sh ./cuda_12.1.1_530.30.02_linux.run --toolkit --silent --installpath=${CUDA_HOME}
rm /tmp/cuda-installer.log


# Get the cuDNN toolkit separately and transfer to your location. You must log in
# on your NVIDIA account at nvidia.com to get access to the download. Now install.
wget -O cudnn-linux-x86_64-8.9.4.25_cuda12-archive.tar.xz "https://liuonline-my.sharepoint.com/:u:/g/personal/xuagu37_liu_se/ESxVoTgY7fNBhy82o1PzpawB80Xu8MNX_YgqqAzck6M6Mg?e=g7KrzS&download=1"
tar xf cudnn-linux-x86_64-8.9.4.25_cuda12-archive.tar.xz
cp cudnn-linux-x86_64-8.9.4.25_cuda12-archive/include/cudnn*.h ${CUDA_HOME}/include/
cp -P cudnn-linux-x86_64-8.9.4.25_cuda12-archive/lib/libcudnn* ${CUDA_HOME}/lib64/
chmod a+r ${CUDA_HOME}/include/cudnn*.h ${CUDA_HOME}/lib64/libcudnn*
chmod a+r ${CUDA_HOME}/include/cudnn*.h ${CUDA_HOME}/lib64/libcudnn*

# Install the NCCL communication libs and headers.The dist tarball file must be
# downloaded separately at https://developer.download.nvidia.com/compute/redist/nccl/
wget -O nccl_2.18.3-1+cuda12.1_x86_64.txz "https://liuonline-my.sharepoint.com/:u:/g/personal/xuagu37_liu_se/EWybqciYeIpOpoCnvRwetsoBopAKJqFbs6CWkaSe8RuqTg?e=3GFHmL&download=1"
tar xf nccl_2.18.3-1+cuda12.1_x86_64.txz 
cp -r nccl_2.18.3-1+cuda12.1_x86_64/lib/* ${CUDA_HOME}/lib64/
cp -r nccl_2.18.3-1+cuda12.1_x86_64/include/nccl* ${CUDA_HOME}/include/

# Fix the *.pc files for NCCL
cat << EOF > ${INSTALL_DIR}/targets/x86_64-linux/lib/pkgconfig/nccl.pc
prefix=${INSTALL_DIR}
exec_prefix=\${prefix}
libdir=\${exec_prefix}/lib64
includedir=\${prefix}/include

Name: nccl
Description: Optimized primitives for collective multi-GPU communication
Version: 2.18.3
Libs: -L\${libdir} -lnccl
Cflags: -I\${includedir}
EOF
cp ${INSTALL_DIR}/targets/x86_64-linux/lib/pkgconfig/nccl.pc ${INSTALL_DIR}/nccl_2.18.3-1+cuda12.1_x86_64/lib/pkgconfig/

# Should there be a need to remove the NCCL installation, this can be done with
# find ${CUDA_HOME}/ -name "*nccl*" -exec rm '{}' \;

# rm -rf cuda_12.1.1_530.30.02_linux.run cudnn-linux-x86_64-8.9.4.25_cuda12-archive cudnn-linux-x86_64-8.9.4.25_cuda12-archive.tar.xz nccl_2.18.3-1+cuda12.1_x86_64 nccl_2.18.3-1+cuda12.1_x86_64.txz
