#!/bin/bash
set -e

# Load necessary modules
module --force purge
module load gcc/system
module load AutoGen/5.18.16 
module load DejaGnu/1.6.3
module load Expect/5.45.4

# Define directories
INSTALL_DIR=/software/sse/manual/GCC/10.3.0
BUILD_DIR=/proj/nsc_testing/xuan/software/sse/manual/GCC/10.3.0

# Create directories
mkdir -p "${INSTALL_DIR}" "${BUILD_DIR}"

# Clean build directory
cd "${BUILD_DIR}"
rm -rf *

# Download GCC source code if not already present
GCC_TAR=gcc-10.3.0.tar.gz
GCC_URL=ftp.mpi-inf.mpg.de/mirrors/gnu/mirror/gcc.gnu.org/pub/gcc/releases/gcc-10.3.0/${GCC_TAR}

if [ ! -f "${GCC_TAR}" ]; then
    wget "${GCC_URL}"
else
    echo "${GCC_TAR} already exists. Skipping download."
fi

# Extract and prepare for build
tar xf "${GCC_TAR}"
cd gcc-10.3.0/
contrib/download_prerequisites
mkdir -p build
cd build

# Configure the build
../configure --prefix="${INSTALL_DIR}" \
    --enable-lto \
    --enable-bootstrap \
    --enable-languages=c,c++,fortran,lto \
    --disable-multilib \
    --disable-libmudflap \
    --enable-threads=posix \
    --enable-clocale=gnu \
    --enable-libstdcxx-debug \
    --enable-libstdcxx-time=yes \
    --enable-gnu-unique-object \
    --enable-checking=release \
    --enable-__cxa_atexit \
    --with-pic \
    --with-system-zlib

# Compile and install
make -j 32
if [ $? -ne 0 ]; then
    echo "Make failed"
    exit 1
fi

make install

# Install additional libraries
for lib in mpfr mpc isl gmp; do
    if [ -d "../$lib" ]; then
        cd "../$lib" && make install
        cd -
    else
        echo "Directory ../$lib not found"
    fi
done

echo "GCC 10.3.0 installation completed successfully."

# export LIBRARY_PATH=/software/sse/manual/GCC/10.3.0/lib:/software/sse/manual/GCC/10.3.0/lib64:$LIBRARY_PATH CPATH=/software/sse/manual/GCC/10.3.0/include:$CPATH PKG_CONFIG_PATH=/software/sse/manual/GCC/10.3.0/lib/pkgconfig:$PKG_CONFIG_PATH
# make -k -j 32 check |& tee ../../check.log


