#!/bin/bash
set -e

module --force purge
module load gcc/system
module load AutoGen/5.18.16 
module load DejaGnu/1.6.3
module load Expect/5.45.4

export INSTALL_DIR=/software/sse/manual/GCC/12.3.0
export BUILD_DIR=/proj/nsc_testing/xuan/software/sse/manual/GCC/12.3.0
mkdir -p ${INSTALL_DIR} ${BUILD_DIR}
cd ${BUILD_DIR}

wget -nc ftp.mpi-inf.mpg.de/mirrors/gnu/mirror/gcc.gnu.org/pub/gcc/releases/gcc-12.3.0/gcc-12.3.0.tar.gz
tar xf gcc-12.3.0.tar.gz && cd gcc-12.3.0/
contrib/download_prerequisites
mkdir -p build && cd build

../configure --prefix=${INSTALL_DIR} \
--enable-lto --enable-bootstrap --enable-languages=c,c++,fortran,lto \
--disable-multilib --disable-libmudflap --enable-threads=posix \
--enable-clocale=gnu --enable-libstdcxx-debug --enable-libstdcxx-time=yes \
--enable-gnu-unique-object --enable-checking=release \
--enable-__cxa_atexit --with-pic --with-system-zlib

make -j 32 && echo $?

## Fix for MPFR, not working for now
cd mpfr && make install
cd ../mpc && make install
cd ../isl && make install
cd ../gmp && make install
cd .. && make install
export LIBRARY_PATH=/software/sse/manual/GCC/12.3.0/lib:/software/sse/manual/GCC/12.3.0/lib64:$LIBRARY_PATH CPATH=/software/sse/manual/GCC/12.3.0/include:$CPATH PKG_CONFIG_PATH=/software/sse/manual/GCC/12.3.0/lib/pkgconfig:$PKG_CONFIG_PATH
make -k check |& tee check2.log
make -j 32 -k check |& tee ../../check2.log

make -k -j 32 check |& tee ../../check.log
make install

# Checking the test log. Filtering out asan (address sanitizer), guality and i386. 
# These errors were deemed harmless based on various internet sources. 
# Guality for instance is very sensitive to the gdb version, which in our case is very old (see https://gcc.gnu.org/ml/gcc-help/2014-01/msg00041.html).
-------------8<----------------------
grep ^FAIL ../../check2.log | grep -vE '\/asan|guality|i386\/'
FAIL: tversion
FAIL: gcc.c-torture/execute/fprintf-2.c   -O3 -g  (test for excess errors)
FAIL: gcc.c-torture/execute/printf-2.c   -O3 -g  (test for excess errors)
FAIL: gcc.c-torture/execute/user-printf.c   -O3 -g  (test for excess errors)
FAIL: g++.dg/tsan/pthread_cond_clockwait.C   -O0  (test for excess errors)
FAIL: g++.dg/tsan/pthread_cond_clockwait.C   -O2  (test for excess errors)
-------------8<----------------------

# # The tversion error log is

# ERROR! The versions of gmp.h (6.2.1) and libgmp (6.1.2) do not match.
# The possible causes are:
#   * A bad configuration in your include/library search paths.
#   * An inconsistency in the include/library search paths of
#     your development environment; an example:
#       https://gcc.gnu.org/legacy-ml/gcc-help/2010-11/msg00359.html
#   * GMP has been upgraded after the first "make check".
#     In such a case, try again after a "make clean".
#   * A new or non-standard version naming is used in GMP.
#     In this case, a patch may already be available on the
#     MPFR web site.  Otherwise please report the problem.
# In the first two cases, this may lead to errors, in particular with MPFR.
# If some other tests fail, please solve that problem first.

# This is caused by the gmp-devel (v. 6.0.0) package on the login nodes. The
# error dissappears on a cluster node after recompiling tversion on it and a
# "make check-mpfr".

grep Error ../../check2.log
make[2]: *** [Makefile:177: check] Error 2
make[1]: *** [Makefile:4085: check-fixincludes] Error 2
make[4]: *** [Makefile:993: tacos] Error 1
make[4]: *** [Makefile:997: tacosh] Error 1
make[4]: *** [Makefile:1021: tasin] Error 1
make[4]: *** [Makefile:1025: tasinh] Error 1
make[4]: *** [Makefile:1053: tdiv] Error 1
make[4]: *** [Makefile:1081: tfma] Error 1
make[4]: *** [Makefile:1085: tfr_div] Error 1
make[4]: *** [Makefile:1113: tmul] Error 1
make[4]: *** [Makefile:1149: tpow] Error 1
make[4]: *** [Makefile:1153: tpow_d] Error 1
make[4]: *** [Makefile:1157: tpow_fr] Error 1
make[4]: *** [Makefile:1161: tpow_ld] Error 1
make[4]: *** [Makefile:1165: tpow_si] Error 1
make[4]: *** [Makefile:1169: tpow_ui] Error 1
make[4]: *** [Makefile:1173: tpow_z] Error 1
make[4]: *** [Makefile:1193: trootofunity] Error 1
make[4]: *** [Makefile:1213: tsqr] Error 1
make[4]: *** [Makefile:1245: ttan] Error 1
make[4]: *** [Makefile:1249: ttanh] Error 1
make[4]: *** [Makefile:1253: tui_div] Error 1
make[4]: *** [Makefile:985: exceptions] Error 1
make[3]: *** [Makefile:2113: check-am] Error 2
make[2]: *** [Makefile:471: check-recursive] Error 1
make[1]: *** [Makefile:8530: check-mpc] Error 2
make[5]: *** [Makefile:2830: test-suite.log] Error 1
make[4]: *** [Makefile:2938: check-TESTS] Error 2
make[3]: *** [Makefile:4286: check-am] Error 2
make[2]: *** [Makefile:500: check-recursive] Error 1
make[1]: *** [Makefile:7471: check-mpfr] Error 2
make: *** [Makefile:2574: do-check] Error 2
