#!/bin/bash
set -e

mkdir -p /software/sse/manual/gcc/9.3.0/nsc1
cd /software/sse/manual/gcc/9.3.0/nsc1

wget ftp://ftp.mpi-sb.mpg.de/pub/gnu/mirror/gcc.gnu.org/pub/gcc/releases/gcc-9.3.0/gcc-9.3.0.tar.gz
tar xf gcc-9.3.0.tar.gz
cd gcc-9.3.0/
contrib/download_prerequisites
mkdir build
cd build
module load gcc/system
../configure --prefix=/software/sse/manual/gcc/9.3.0 \
--enable-lto --enable-bootstrap --enable-languages=c,c++,fortran,lto \
--disable-multilib --disable-libmudflap --enable-threads=posix \
--enable-clocale=gnu --enable-libstdcxx-debug --enable-libstdcxx-time=yes \
--enable-gnu-unique-object --enable-checking=release \
--enable-__cxa_atexit --with-pic --with-system-zlib
make && echo $?

make -k check > check.log 2>&1
make install

# Checking the test log. Filtering out asan (address sanitizer), guality and
# i386. These errors were deemed harmless based on various internet sources. Guality
# for instance is very sensitive to the gdb version, which in our case is very
# old (see https://gcc.gnu.org/ml/gcc-help/2014-01/msg00041.html).
-------------8<----------------------
grep ^FAIL check.log | grep -vE '\/asan|guality|i386\/'
FAIL: g++.dg/tls/thread_local-order2.C  -std=c++14 execution test
FAIL: g++.dg/tls/thread_local-order2.C  -std=c++17 execution test
FAIL: tversion
FAIL: 27_io/filesystem/operations/last_write_time.cc execution test
FAIL: experimental/filesystem/operations/last_write_time.cc execution test
FAIL: libstdc++-prettyprinters/80276.cc whatis p4
FAIL: libstdc++-prettyprinters/cxx17.cc print path2
FAIL: libstdc++-prettyprinters/80276.cc whatis p4
FAIL: libstdc++-prettyprinters/cxx17.cc print path2
-------------8<----------------------

# The first two tls fails are likely caused by an old glibc, see
# https://gcc.gnu.org/bugzilla/show_bug.cgi?id=79427
# The tversion error log is
	ERROR! The versions of gmp.h (6.1) and libgmp (6.0.0) do not match.
	The possible causes are:
	  * A bad configuration in your include/library search paths.
	  * An inconsistency in the include/library search paths of
		your development environment; an example:
		  http://gcc.gnu.org/ml/gcc-help/2010-11/msg00359.html
	  * GMP has been upgraded after the first "make check".
		In such a case, try again after a "make clean".
	  * A new or non-standard version naming is used in GMP.
		In this case, a patch may already be available on the
		MPFR web site.  Otherwise please report the problem.
	In the first two cases, this may lead to errors, in particular with MPFR.
	If some other tests fail, please solve that problem first.
	FAIL tversion (exit status: 1)
# This is caused by the gmp-devel (v. 6.0.0) package on the login nodes. The
# error dissappears on a cluster node after recompiling tversion on it and a
# "make check-mpfr".
#
# The remaining failures seem related to
# https://gcc.gnu.org/bugzilla/show_bug.cgi?id=86756 and
# https://gcc.gnu.org/bugzilla/show_bug.cgi?id=83906 where gdb is mentioned.
# Hard to tell whether these errors concerning pretty-printing are worth
# spending more efforts on, but I believe not, and will leave it as is.
