module --force purge 
module load Mambaforge/23.3.1-1-hpc1-bdist
module load gcc/system

export INSTALL_DIR=/software/sse/manual/HelixFold3/73cd80b

mkdir -p ${INSTALL_DIR}

# To download HelixFold3
cd ${INSTALL_DIR}
git clone https://github.com/PaddlePaddle/PaddleHelix.git && cd PaddleHelix
git checkout 73cd80b
mv apps/protein_folding/helixfold3/* ${INSTALL_DIR}/
rm -rf ${INSTALL_DIR}/PaddleHelix
ln -s ${INSTALL_DIR}/inference.py ${INSTALL_DIR}/scripts/inference.py 
ln -s ${INSTALL_DIR}/run_infer.sh ${INSTALL_DIR}/scripts/run_infer.sh
chmod +x ${INSTALL_DIR}/inference.py ${INSTALL_DIR}/run_infer.sh

# Install py env
mamba create -p ${INSTALL_DIR}/envs/helixfold3_73cd80b python=3.9
mamba activate ${INSTALL_DIR}/envs/helixfold3_73cd80b
mamba install -y -c bioconda aria2 hmmer==3.3.2 kalign2==2.04 hhsuite==3.3.0  
mamba install -y openbabel  
mamba install -y cudatoolkit=11.7.1 cudnn=8.4.0 nccl=2.14.3

# install paddlepaddle
# python3 -m pip install paddlepaddle-gpu==2.6.1.post120 -f https://www.paddlepaddle.org.cn/whl/linux/mkl/avx/stable.html
# or lower version: https://paddle-wheel.bj.bcebos.com/2.5.1/linux/linux-gpu-cuda11.7-cudnn8.4.1-mkl-gcc8.2-avx/paddlepaddle_gpu-2.5.1.post117-cp39-cp39-linux_x86_64.whl
python3 -m pip install paddlepaddle-gpu==2.5.1.post117 -f https://paddle-wheel.bj.bcebos.com/2.5.1/linux/linux-gpu-cuda11.7-cudnn8.4.1-mkl-gcc8.2-avx/paddlepaddle_gpu-2.5.1.post117-cp39-cp39-linux_x86_64.whl

# Install requirements
python3 -m pip install -r requirements.txt

# Install MAXIT
wget https://sw-tools.rcsb.org/apps/MAXIT/maxit-v11.100-prod-src.tar.gz
tar -xzvf maxit-v11.100-prod-src.tar.gz
rm maxit-v11.100-prod-src.tar.gz
cd maxit-v11.100-prod-src
# Modify etc/platform.sh
# Line 70: gcc_ver=`gcc --version | grep -e " 4\."`
# to: gcc_ver=`gcc --version | grep -e " 8\."`
make binary

# Download parameters and data
mkdir -p ${INSTALL_DIR}/init_models && cd ${INSTALL_DIR}/init_models && wget https://paddlehelix.bd.bcebos.com/HelixFold3/params/HelixFold3-params-240814.zip
unzip HelixFold3-params-240814.zip
mv HelixFold3-params-240814/HelixFold3-240814.pdparams ${INSTALL_DIR}/init_models/
rm -rf HelixFold3-params-240814* __MACOSX

export ALPHAFOLD_DB=/proj/common-datasets/AlphaFold
aria2c https://paddlehelix.bd.bcebos.com/HelixFold3/CCD/ccd_preprocessed_etkdg.pkl.gz --dir=${ALPHAFOLD_DB}
aria2c https://paddlehelix.bd.bcebos.com/HelixFold3/MSA/Rfam-14.9_rep_seq.fasta --dir=${ALPHAFOLD_DB}
aria2c https://ftp.ebi.ac.uk/pub/databases/metagenomics/peptide_database/2018_12/mgy_clusters.fa.gz --dir=${ALPHAFOLD_DB}
gunzip ${ALPHAFOLD_DB}/mgnify_2018_12/mgy_clusters.fa.gz
mv ${ALPHAFOLD_DB}/mgnify_2018_12/mgy_clusters.fa ${ALPHAFOLD_DB}/mgnify_2018_12/mgy_clusters_2018_12.fa

# Patch: added flag --run_feature_only
# Copy 73cd80b-inference.py to inference.py
# Copy 73cd80b-run_infer.sh to run_infer.sh