module load Mambaforge/23.3.1-1-hpc1-bdist

export INSTALL_DIR=/software/sse/manual/LocalColabFold/1.5.5
mkdir -p ${INSTALL_DIR}

mamba create -p ${INSTALL_DIR}/envs/localcolabfold_1.5.5 -c conda-forge -c bioconda \
    git python=3.10 openmm==7.7.0 pdbfixer \
    kalign2=2.04 hhsuite=3.3.0 mmseqs2=15.6f452 -y

mamba activate ${INSTALL_DIR}/envs/localcolabfold_1.5.5

# install ColabFold and Jaxlib
"${INSTALL_DIR}/envs/localcolabfold_1.5.5/bin/pip" install --no-warn-conflicts "colabfold[alphafold-without-jax] @ git+https://github.com/sokrypton/ColabFold"
"${INSTALL_DIR}/envs/localcolabfold_1.5.5/bin/pip" install --upgrade "jax[cuda11_pip]==0.4.23" -f https://storage.googleapis.com/jax-releases/jax_cuda_releases.html
"${INSTALL_DIR}/envs/localcolabfold_1.5.5/bin/pip" install "colabfold[alphafold]"
"${INSTALL_DIR}/envs/localcolabfold_1.5.5/bin/pip" list | grep "^nvidia-" | awk '{print $1}' | xargs "${INSTALL_DIR}/envs/localcolabfold_1.5.5/bin/pip" uninstall -y
"${INSTALL_DIR}/envs/localcolabfold_1.5.5/bin/pip" install --upgrade tensorflow[and-cuda]
"${INSTALL_DIR}/envs/localcolabfold_1.5.5/bin/pip" install silence_tensorflow

cd "${INSTALL_DIR}/envs/localcolabfold_1.5.5/lib/python3.10/site-packages/colabfold"
# Use 'Agg' for non-GUI backend
sed -i -e "s#from matplotlib import pyplot as plt#import matplotlib\nmatplotlib.use('Agg')\nimport matplotlib.pyplot as plt#g" plot.py
# modify the default params directory
sed -i -e "s#appdirs.user_cache_dir(__package__ or \"colabfold\")#\"${COLABFOLDDIR}/colabfold\"#g" download.py
# suppress warnings related to tensorflow
sed -i -e "s#from io import StringIO#from io import StringIO\nfrom silence_tensorflow import silence_tensorflow\nsilence_tensorflow()#g" batch.py
# remove cache directory
rm -rf __pycache__


# https://github.com/google/jax/issues/18027
# >>> import jax
# >>> print(jax.local_devices()[0].platform)
# CUDA backend failed to initialize: Found CUDA version 11070, but JAX was built against version 11080, which is newer. The copy of CUDA that is installed must be at least as new as the version against which JAX was built. (Set TF_CPP_MIN_LOG_LEVEL=0 and rerun for more info.)
# Solution
# "${INSTALL_DIR}/envs/localcolabfold_1.5.5/bin/pip" install --upgrade tensorflow[and-cuda]