#!/bin/bash
# NSC Condaforge pip wrapper
# (c) NSC, Rickard Armiento, 2021

if [ -z "NSC_CONDAFORGE_PATH" ]; then
    exec "$0" "$@"
fi

EXECNAME="$(basename "$0")"

if [ -n "$CONDA_DEFAULT_ENV" -a "$CONDA_DEFAULT_ENV" != "base" ]; then

    # If we are in an active conda environment where the pip command (pip/pip2/pip3) the user asks for, just run it
    if [ -e "${CONDA_PREFIX}/bin/$EXECNAME" ]; then
	exec "${CONDA_PREFIX}/bin/$EXECNAME"
    fi

    # Educate the user about prefering conda channels
    echo "" >&2
    echo "## NSC note ##"
    echo "You are trying to run $EXECNAME in a condaforge environment that does not have that command." >&2
    echo "Before installing any software with pip, please check if the same software is available via one" >&2
    echo "of the existing channels:" >&2
    echo "- by running, e.g.,: mamba search -c conda-forge name_of_package" >&2
    echo "- or by using the online conda package search here: https://anaconda.org/search" >&2
    echo "" >&2
    if [ -e "${CONDA_PREFIX}/bin/pip" ]; then
	# If the active conda environment has 'pip', point them to that
	echo "Your conda environment supplies the command 'pip' without a version number." >&2
	echo "Hence, if you need to use pip to install the software, use that command." >&2
    else
	# If the active conda environment does not have 'pip', help them get an environment with pip
	echo "If you still need to install software with pip," >&2
	echo "you are recommended to re-create a new environment with pip included." >&2
	echo "You can try to copy the present environment and add pip as follows" >&2
	echo "(substitute 'newenv' with the new name to use):" >&2
	echo "-------------------------------------------------------------" >&2
	echo "              mamba list -e > env_reqs.txt" >&2
	echo "              mamba create -n newenv --file env_reqs.txt pip" >&2
	echo "              mamba activate newenv" >&2
	echo "-------------------------------------------------------------" >&2
	echo ")" >&2
	echo "(NSC generally does NOT RECOMMENDED adding packages to existing conda environements, since it may" >&2
	echo "alter them unexpectedly, e.g., by downgrading library versions or even the version of python" >&2
        echo "itself. Nevertheless, if you are confident that you can handle this, you can instead alternatively" >&2
	echo "try to add pip to the present environment by:" >&2
	echo "  mamba install pip" >&2
	echo ")" >&2
    fi
    echo >&2
    echo "$EXECNAME: command not found" >&2
    exit 1
fi

echo "## NSC note ##"
echo "You are trying to run $EXECNAME while having an NSC Mambaforge module loaded, but while not having" >&2
echo "an activated environment. Please first activate the environment you want to use, e.g., by:" >&2
echo >&2
echo "  mamba activate envname" >&2
echo >&2
echo "(It is generally NOT RECOMMENDED to use pip outside of an active environment."
echo "However, if you are sure this is what you want to do, call the pip binary you want to use,"
echo "e.g., the system pip, using its full absolute path.)"
echo >&2
echo "$EXECNAME: command not found" >&2
exit 1
