# NB! This build.txt can be run as a script with "bash -x build.txt 2>&1 | tee
# make.log". Just be sure to set variables appropriately.
#
# Get the source. Unpack it and 'cd' to the unpacked dir.
# wget https://github.com/xianyi/OpenBLAS/archive/v0.3.13.tar.gz
# tar xf v0.3.19.tar.gz && cd OpenBLAS-0.3.19
#module load gcc/system
export BASE_INSTALLDIR=/software/sse/manual/OpenBLAS/0.3.19/g83/nsc1

# Set the number of parallel make processes and whichever architecture you
# want. On this OpenBLAS release Zen (ZEN) is supported, so we choose it for
# Berzelius. HASWELL could in principle also be chosen and is not terrible in
# terms of performance.
export NUM_PAR_MAKE=16 ARCH=ZEN

# Basic build options. No need to fiddle with these usually.
BASE_OPTS="MAKE_NB_JOBS=${NUM_PAR_MAKE} \
TARGET=${ARCH} \
BINARY=64 \
CC=gcc FC=gfortran F77=gfortran"

# Loop over the OpenBLAS build options to enable/disable feature combinations
# and set descriptive library names accordingly. The "reverse" order of the
# no_lapack option is for technical reason when you install, the cmake and
# pkgconfig dirs gets correctly "filled" this way.
for t in seq thr omp; do
	for no_lapack in 1 0; do
		for do_ilp64 in 0 1; do
			# Blank the previous library name and options
			LIBSUFFIX=""
			EXTRA_OPTS=""
			THREADING_OPTS=""

			# Set correct build options
			if (( no_lapack == 1 )); then
				LIBSUFFIX="blasonly"
			else
				unset LIBSUFFIX
			fi
			
			if [[ "${t}" == "seq" ]]; then
				INSTALLDIR="${BASE_INSTALLDIR}/sequential"
				THREADING_OPTS="USE_THREAD=0 USE_OPENMP=0 USE_LOCKING=1 NO_AFFINITY=1"
			elif [[ "${t}" == "thr" ]]; then
				INSTALLDIR="${BASE_INSTALLDIR}/pthread"

				# Disabling affinity for (posix) threading avoids issues in codes
				# which manage affinity themselves. We disable affinity for the
				# pthread parallel OpenBLAS but not for the OpenMP parallel
				# OpenBLAS. Disabling affinity with OpenMP parallel OpenBLAS is not
				# advisable because it messes silently with all attempts to
				# use the OpenMP runtime affinity with unexpected results,
				# e.g. not using parallel threads at all when setting
				# OMP_PROC_BIND and OMP_PLACES affinity controlling environment
				# variables.
				THREADING_OPTS="USE_OPENMP=0 USE_THREAD=1 NO_AFFINITY=1 NUM_THREADS=128"
			else
				INSTALLDIR="${BASE_INSTALLDIR}/openmp"
				THREADING_OPTS="USE_OPENMP=1 NO_AFFINITY=0 NUM_THREADS=128"
			fi

			if (( do_ilp64 == 1 )); then
				INSTALLDIR="${INSTALLDIR}/ilp64"
			else
				INSTALLDIR="${INSTALLDIR}/lp64"
			fi
			
			# Add extra specifications for the build
			EXTRA_OPTS="${THREADING_OPTS} NO_LAPACK=$no_lapack INTERFACE64=$do_ilp64"
			
			# Set the final specification string
			if [[ -z ${LIBSUFFIX} ]]; then
				SPECSTRING="${BASE_OPTS} ${EXTRA_OPTS}"
			else
				SPECSTRING="${BASE_OPTS} ${EXTRA_OPTS} LIBNAMESUFFIX=${LIBSUFFIX}"
			fi

			# Do the compilation
			make ${SPECSTRING}
			if [[ "$?" == "0" ]]; then
				make ${SPECSTRING} PREFIX=${INSTALLDIR} install && \
				make ${SPECSTRING} clean
			else
				echo "FAILED: make ${SPECSTRING}"
				exit 1
			fi
			#echo "make ${SPECSTRING}"
			#echo "make ${SPECSTRING} PREFIX=${INSTALLDIR} install"
			#echo "make ${SPECSTRING} clean"
		done
	done
done

# Testing was done manually on sequential, pthread (OPENBLAS_NUM_THREADS=4) and
# openmp (OMP_NUM_THREADS=4) for lp64 by replicating the build flags for each
# case above and using the 'lapack-test' make target. The sequential test
# showed 0 errors while pthread and openmp both showed:
#
# 			-->   LAPACK TESTING SUMMARY  <--
# SUMMARY             	nb test run 	numerical error   	other error  
# ================   	===========	=================	================  
# REAL             	1304841		4	(0.000%)	0	(0.000%)	
# DOUBLE PRECISION	1316957		0	(0.000%)	0	(0.000%)	
# COMPLEX          	768540		1	(0.000%)	0	(0.000%)	
# COMPLEX16         	777128		0	(0.000%)	0	(0.000%)	
# 
# --> ALL PRECISIONS	4167466		5	(0.000%)	0	(0.000%)	
#
# This is likely so minor it can be ignored, using low precision implies you
# accept the risks (in addition). All builds of OpenBLAS by default also run
# the make target 'blas-test' which runs simple linalg ops for BLAS1, BLAS2 and
# BLAS3, and these tests have passed for all OpenBLAS installations here.

# Set up some links which help apps find reasonable defaults for libblas and
# liblapack.
for i in /software/sse/manual/OpenBLAS/0.3.19/g83/nsc1/{sequential,openmp,pthread}/{lp64,ilp64}/lib; \
	do cd $i;\
	ln -s libopenblas.so liblapack.so;\
	ln -s libopenblas_blasonly.so libblas.so;\
done






module rm mpprun/4.1.5
module load gcc/system

export BASE_INSTALL_DIR=/software/sse/manual/OpenBLAS/0.3.19/gcc-8.5.0
export BUILD_DIR=/proj/nsc_testing/xuan/software/sse/manual/OpenBLAS/0.3.19/gcc-8.5.0
mkdir -p ${INSTALL_DIR} ${BUILD_DIR}
cd ${BUILD_DIR}

# Get the source. Unpack it and 'cd' to the unpacked dir.
wget -nc https://github.com/xianyi/OpenBLAS/archive/v0.3.19.tar.gz
tar xf v0.3.19.tar.gz && cd OpenBLAS-0.3.19