module rm mpprun/4.1.5
module load GCC/.12.3.0

export BASE_INSTALL_DIR=/software/sse/manual/OpenBLAS/0.3.23/gcc-12.3.0
export BUILD_DIR=/proj/nsc_testing/xuan/software/sse/manual/OpenBLAS/0.3.23/gcc-12.3.0
mkdir -p ${BASE_INSTALL_DIR} ${BUILD_DIR}
cd ${BUILD_DIR}

# Get the source. Unpack it and 'cd' to the unpacked dir.
wget https://github.com/xianyi/OpenBLAS/archive/v0.3.23.tar.gz
tar xf v0.3.23.tar.gz && cd OpenBLAS-0.3.23

sed -i 's|#! /usr/bin/env python|#! /usr/bin/env python3|' "${BUILD_DIR}/OpenBLAS-0.3.23/lapack-netlib/lapack_testing.py"

# sequential
SPECSTRING="MAKE_NB_JOBS=16 TARGET=ZEN BINARY=64 CC=gcc FC=gfortran F77=gfortran USE_THREAD=0 USE_OPENMP=0 USE_LOCKING=1 NO_AFFINITY=1 NO_LAPACK=0 INTERFACE64=0"
make $SPECSTRING
make $SPECSTRING lapack-test |& tee ../lapack-test-sequential.txt
INSTALL_DIR=${BASE_INSTALL_DIR}/sequential/lp64
make ${SPECSTRING} PREFIX=${INSTALL_DIR} install
make $SPECSTRING clean

# pthread
SPECSTRING="MAKE_NB_JOBS=16 TARGET=ZEN BINARY=64 CC=gcc FC=gfortran F77=gfortran USE_OPENMP=0 USE_THREAD=1 NO_AFFINITY=1 NUM_THREADS=128 NO_LAPACK=0 INTERFACE64=0"
make $SPECSTRING
make $SPECSTRING OPENBLAS_NUM_THREADS=4 lapack-test |& tee ../lapack-test-pthread.txt
INSTALL_DIR=${BASE_INSTALL_DIR}/pthread/lp64
make ${SPECSTRING} PREFIX=${INSTALL_DIR} install
make $SPECSTRING clean

# openmp
SPECSTRING="MAKE_NB_JOBS=16 TARGET=ZEN BINARY=64 CC=gcc FC=gfortran F77=gfortran USE_OPENMP=1 NO_AFFINITY=0 NUM_THREADS=128 NO_LAPACK=0 INTERFACE64=0"
make $SPECSTRING
make $SPECSTRING OMP_NUM_THREADS=4 lapack-test |& tee ../lapack-test-openmp.txt
INSTALL_DIR=${BASE_INSTALL_DIR}/openmp/lp64
make ${SPECSTRING} PREFIX=${INSTALL_DIR} install
make $SPECSTRING clean


# Testing was done manually on sequential, pthread (OPENBLAS_NUM_THREADS=4) and
# openmp (OMP_NUM_THREADS=4) for lp64 by replicating the build flags for each
# case above and using the 'lapack-test' make target. The sequential test
# showed 0 errors while pthread and openmp both showed:
#
# -->   LAPACK TESTING SUMMARY  <--
# SUMMARY             	nb test run 	numerical error   	other error  
# ================   	===========	=================	================  
# REAL             	1304841		4	(0.000%)	0	(0.000%)	
# DOUBLE PRECISION	1316957		0	(0.000%)	0	(0.000%)	
# COMPLEX          	768540		1	(0.000%)	0	(0.000%)	
# COMPLEX16         	777128		0	(0.000%)	0	(0.000%)	
# 
# --> ALL PRECISIONS	4167466		5	(0.000%)	0	(0.000%)	
#
# This is likely so minor it can be ignored, using low precision implies you
# accept the risks (in addition). All builds of OpenBLAS by default also run
# the make target 'blas-test' which runs simple linalg ops for BLAS1, BLAS2 and
# BLAS3, and these tests have passed for all OpenBLAS installations here.

# Set up some links which help apps find reasonable defaults for libblas and
# liblapack.
for i in ${BASE_INSTALL_DIR}/{sequential,openmp,pthread}/lp64/lib; \
	do cd $i;\
	ln -s libopenblas.so liblapack.so;\
	# ln -s libopenblas_blasonly.so libblas.so;\
done



########## Notes ###########

# USE_OPENMP: OpenMP (Open Multi-Processing) is an API that supports multi-threading in code. When USE_OPENMP is enabled during the compilation or configuration of OpenBLAS, it means that OpenBLAS will be built with support for OpenMP. This allows OpenBLAS to utilize multiple CPU cores and run linear algebra operations in parallel, which can significantly improve performance on multi-core systems. Enabling OpenMP is generally a good choice when you have a multi-core CPU and want to take advantage of parallel processing.

# USE_THREAD: The USE_THREAD option is related to multi-threading but is not specific to OpenMP. When USE_THREAD is enabled, it means that OpenBLAS is built with support for multi-threading in general. OpenBLAS can use its own internal threading mechanisms or rely on system-specific threading libraries like POSIX threads (pthread). Enabling USE_THREAD is another way to parallelize linear algebra operations on multi-core systems.