
module rm mpprun/4.1.5
module load GCC/.13.2.0

export BASE_INSTALL_DIR=/software/sse/manual/OpenBLAS/0.3.25/gcc-13.2.0
export BUILD_DIR=/proj/nsc_testing/xuan/software/sse/manual/OpenBLAS/0.3.25/gcc-13.2.0
mkdir -p ${INSTALL_DIR} ${BUILD_DIR}
cd ${BUILD_DIR}

# Get the source. Unpack it and 'cd' to the unpacked dir.
wget -nc https://github.com/xianyi/OpenBLAS/archive/v0.3.25.tar.gz
tar xf v0.3.25.tar.gz && cd OpenBLAS-0.3.25

# ${BUILD_DIR}/OpenBLAS-0.3.25/lapack-netlib/lapack_testing.py
# change #! /usr/bin/env python
# to #! /usr/bin/env python3
sed -i 's|#! /usr/bin/env python|#! /usr/bin/env python3|' "${BUILD_DIR}/OpenBLAS-0.3.25/lapack-netlib/lapack_testing.py"

# Set the number of parallel make processes and whichever architecture you
# want. On this OpenBLAS release Zen (ZEN) is supported, so we choose it for
# Berzelius. HASWELL could in principle also be chosen and is not terrible in
# terms of performance.
export NUM_PAR_MAKE=16 ARCH=ZEN

# Basic build options. No need to fiddle with these usually.
BASE_OPTS="MAKE_NB_JOBS=${NUM_PAR_MAKE} \
TARGET=${ARCH} \
BINARY=64 \
CC=gcc FC=gfortran F77=gfortran"

# Loop over the OpenBLAS build options to enable/disable feature combinations
# and set descriptive library names accordingly. The "reverse" order of the
# no_lapack option is for technical reason when you install, the cmake and
# pkgconfig dirs gets correctly "filled" this way.
for t in seq thr omp; do
# for t in seq; do
	for no_lapack in 0; do
		for do_ilp64 in 0; do

			# Blank the previous library name and options
			LIBSUFFIX=""
			EXTRA_OPTS=""
			THREADING_OPTS=""

			# Set correct build options
			if (( no_lapack == 1 )); then
				LIBSUFFIX="blasonly"
			else
				unset LIBSUFFIX
			fi
			
			if [[ "${t}" == "seq" ]]; then
				INSTALL_DIR="${BASE_INSTALL_DIR}/sequential"
				THREADING_OPTS="USE_THREAD=0 USE_OPENMP=0 USE_LOCKING=1 NO_AFFINITY=1"
			elif [[ "${t}" == "thr" ]]; then
				INSTALL_DIR="${BASE_INSTALL_DIR}/pthread"

				# Disabling affinity for (posix) threading avoids issues in codes
				# which manage affinity themselves. We disable affinity for the
				# pthread parallel OpenBLAS but not for the OpenMP parallel
				# OpenBLAS. Disabling affinity with OpenMP parallel OpenBLAS is not
				# advisable because it messes silently with all attempts to
				# use the OpenMP runtime affinity with unexpected results,
				# e.g. not using parallel threads at all when setting
				# OMP_PROC_BIND and OMP_PLACES affinity controlling environment
				# variables.
				THREADING_OPTS="USE_OPENMP=0 USE_THREAD=1 NO_AFFINITY=1 NUM_THREADS=128"
			else
				INSTALL_DIR="${BASE_INSTALL_DIR}/openmp"
				THREADING_OPTS="USE_OPENMP=1 NO_AFFINITY=0 NUM_THREADS=128"
			fi

			if (( do_ilp64 == 1 )); then
				INSTALL_DIR="${INSTALL_DIR}/ilp64"
			else
				INSTALL_DIR="${INSTALL_DIR}/lp64"
			fi
			
			# Add extra specifications for the build
			EXTRA_OPTS="${THREADING_OPTS} NO_LAPACK=$no_lapack INTERFACE64=$do_ilp64"
			
			# Set the final specification string
			if [[ -z ${LIBSUFFIX} ]]; then
				SPECSTRING="${BASE_OPTS} ${EXTRA_OPTS}"
			else
				SPECSTRING="${BASE_OPTS} ${EXTRA_OPTS} LIBNAMESUFFIX=${LIBSUFFIX}"
			fi

			# Do the compilation
			# make ${SPECSTRING}
			echo make ${SPECSTRING} |& tee ../SPECSTRING-$t.txt
			if [[ "$?" == "0" ]]; then
				# make ${SPECSTRING} PREFIX=${INSTALL_DIR} install

				# if [[ "${t}" == "seq" ]]; then
				# 	make TARGET=ZEN lapack-test |& tee ../lapack-test-$t.txt
				# elif [[ "${t}" == "thr" ]]; then
				# 	make TARGET=ZEN OPENBLAS_NUM_THREADS=4 lapack-test |& tee ../lapack-test-$t.txt
				# else
				# 	make TARGET=ZEN OMP_NUM_THREADS=4 lapack-test |& tee ../lapack-test-$t.txt
				# fi
				
				make ${SPECSTRING} clean
			else
				echo "FAILED: make ${SPECSTRING}"
				exit 1
			fi
		done
	done
done


# Testing was done manually on sequential, pthread (OPENBLAS_NUM_THREADS=4) and
# openmp (OMP_NUM_THREADS=4) for lp64 by replicating the build flags for each
# case above and using the 'lapack-test' make target. The sequential test
# showed 0 errors while pthread and openmp both showed:
#
# -->   LAPACK TESTING SUMMARY  <--
# SUMMARY             	nb test run 	numerical error   	other error  
# ================   	===========	=================	================  
# REAL             	1304841		4	(0.000%)	0	(0.000%)	
# DOUBLE PRECISION	1316957		0	(0.000%)	0	(0.000%)	
# COMPLEX          	768540		1	(0.000%)	0	(0.000%)	
# COMPLEX16         	777128		0	(0.000%)	0	(0.000%)	
# 
# --> ALL PRECISIONS	4167466		5	(0.000%)	0	(0.000%)	
#
# This is likely so minor it can be ignored, using low precision implies you
# accept the risks (in addition). All builds of OpenBLAS by default also run
# the make target 'blas-test' which runs simple linalg ops for BLAS1, BLAS2 and
# BLAS3, and these tests have passed for all OpenBLAS installations here.

# Set up some links which help apps find reasonable defaults for libblas and
# liblapack.
# for i in ${BASE_INSTALL_DIR}/{sequential,openmp,pthread}/lp64/lib; \
# 	do cd $i;\
# 	ln -s libopenblas.so liblapack.so;\
# 	# ln -s libopenblas_blasonly.so libblas.so;\
# done



########## Notes ###########

# USE_OPENMP: OpenMP (Open Multi-Processing) is an API that supports multi-threading in code. When USE_OPENMP is enabled during the compilation or configuration of OpenBLAS, it means that OpenBLAS will be built with support for OpenMP. This allows OpenBLAS to utilize multiple CPU cores and run linear algebra operations in parallel, which can significantly improve performance on multi-core systems. Enabling OpenMP is generally a good choice when you have a multi-core CPU and want to take advantage of parallel processing.

# USE_THREAD: The USE_THREAD option is related to multi-threading but is not specific to OpenMP. When USE_THREAD is enabled, it means that OpenBLAS is built with support for multi-threading in general. OpenBLAS can use its own internal threading mechanisms or rely on system-specific threading libraries like POSIX threads (pthread). Enabling USE_THREAD is another way to parallelize linear algebra operations on multi-core systems.