module load Mambaforge/23.3.1-1-hpc1-bdist
module load buildenv-gcccuda/11.8.0-gcc11.3.0

export INSTALL_DIR=/software/sse/manual/OpenFold/2.1.0
mkdir -p ${INSTALL_DIR}


# To download OpenFold
wget -O /tmp/v2.1.0.tar.gz https://github.com/aqlaboratory/openfold/archive/refs/tags/v.2.1.0.tar.gz
tar -xf /tmp/v2.1.0.tar.gz -C ${INSTALL_DIR} --strip-components=1

# We create a conda env from a yml file.
cd ${INSTALL_DIR}
mamba env create -f ${INSTALL_DIR}/environment.yml -p ${INSTALL_DIR}/envs/openfold_2.1.0

# To fix a path issue
cd ${INSTALL_DIR}/scripts
sed -i 's|scripts/colabfold_search.sh|colabfold_search.sh|g' precompute_alignments_mmseqs.py

cd ${INSTALL_DIR}

# Download folding resources
wget -N --no-check-certificate -P openfold/resources \
    https://git.scicore.unibas.ch/schwede/openstructure/-/raw/7102c63615b64735c4941278d92b554ec94415f8/modules/mol/alg/src/stereo_chemical_props.txt

# Certain tests need access to this file
mkdir -p tests/test_data/alphafold/common
ln -rs openfold/resources/stereo_chemical_props.txt tests/test_data/alphafold/common

# Decompress test data
gunzip -c tests/test_data/sample_feats.pickle.gz > tests/test_data/sample_feats.pickle

conda activate ${INSTALL_DIR}/envs/openfold_2.1.0
python setup.py install

echo "Download CUTLASS, required for Deepspeed Evoformer attention kernel"
git clone https://github.com/NVIDIA/cutlass --depth 1
# conda env config vars set CUTLASS_PATH=$PWD/cutlass

# conda env config vars set KMP_AFFINITY=none

chmod -R 755 ${INSTALL_DIR}