#!/bin/bash
set -e

module --force purge
module load gcc/system
module load CUDA/.12.1.1

export INSTALL_DIR=/software/sse/manual/OpenMPI/4.1.2/gcc-8.5.0/cuda-12.1.1/hpc1
export BUILD_DIR=/proj/nsc_testing/xuan/software/sse/manual/OpenMPI/4.1.2/gcc-8.5.0/cuda-12.1.1/hpc1
mkdir -p ${INSTALL_DIR} ${BUILD_DIR}
cd ${BUILD_DIR}


wget https://download.open-mpi.org/release/open-mpi/v4.1/openmpi-4.1.2.tar.gz
tar xf openmpi-4.1.2.tar.gz && cd openmpi-4.1.2

# On a compute node
./configure \
--prefix=${INSTALL_DIR} \
--with-hwloc=/usr --with-libevent=/usr \
--enable-mpi1-compatibility --without-xpmem --with-cma \
--without-knem --with-slurm --enable-shared --with-pic \
--with-pmix --with-cuda=${CUDA_HOME} \
--with-singularity --with-ucx \
CC=gcc CXX=g++ FC=gfortran F77=gfortran \
LDFLAGS="-L${CUDA_HOME}/lib64 -Wl,-rpath,${CUDA_HOME}/lib64" \
CPPFLAGS="-I${CUDA_HOME}/include"

make -j32
make check |& tee ../check.log

# On login node
make install

# Add these lines to ${INSTALL_DIR}/etc/openmpi-mca-params.conf
cat <<EOF >> ${INSTALL_DIR}/etc/openmpi-mca-params.conf
btl = ^openib
coll_hcoll_enable = 0
rmaps_base_mapping_policy = numa:SPAN
hwloc_base_binding_policy = core
rmaps_base_ranking_policy = numa:FILL
EOF



# # OSU
# module load OpenMPI/.4.1.2-gcc-8.5.0-cuda-12.1.1
# wget http://mvapich.cse.ohio-state.edu/download/mvapich/osu-micro-benchmarks-6.1.tar.gz
# tar xf osu-micro-benchmarks-6.1.tar.gz && cd osu-micro-benchmarks-6.1
# ./configure --enable-cuda --enable-ncclomb --with-nccl=${CUDA_HOME} --with-cuda=${CUDA_HOME} --prefix=/software/sse/manual/OSU/6.1/gcc-8.5.0/cuda-12.1.1/ompi416 CC=mpicc CXX=mpicxx

# make 
# /software/sse/manual/CUDA/12.1.1_530.30.02/include/crt/host_config.h:132:2: error: #error -- unsupported GNU version! gcc versions later than 12 are not supported! The nvcc flag '-allow-unsupported-compiler' can be used to override this version check; however, using an unsupported host compiler may cause compilation failure or incorrect run time execution. Use at your own risk.