#!/bin/bash
set -euo pipefail

# Configurable version
VSCODE_SERVER_VERSION="4.99.3"
TARBALL_NAME="code-server-${VSCODE_SERVER_VERSION}-linux-amd64.tar.gz"
INSTALL_DIR="/software/sse/manual/VSCode-Server/${VSCODE_SERVER_VERSION}/bdist"
DOWNLOAD_URL="https://github.com/coder/code-server/releases/download/v${VSCODE_SERVER_VERSION}/${TARBALL_NAME}"

echo "🔧 Installing VSCode Server version ${VSCODE_SERVER_VERSION}"
echo "📁 Target install directory: ${INSTALL_DIR}"

# Create and switch to install directory
mkdir -p "${INSTALL_DIR}"
cd "${INSTALL_DIR}"

# Download tarball
echo "⬇️  Downloading from ${DOWNLOAD_URL}"
wget "${DOWNLOAD_URL}"

# Extract tarball (remove top-level directory)
echo "📦 Extracting ${TARBALL_NAME}"
tar --strip-components=1 -xzf "${TARBALL_NAME}"

# Clean up archive
rm -f "${TARBALL_NAME}"

echo "✅ VSCode Server ${VSCODE_SERVER_VERSION} installed successfully in ${INSTALL_DIR}"
