#!/bin/bash
set -euo pipefail

# Set VSCode Server version
VSCODE_VERSION="4.99.3"
TARBALL="code-server-${VSCODE_VERSION}-linux-amd64.tar.gz"
INSTALL_DIR="/software/sse2/tetralith_el9/manual/VSCode-Server/${VSCODE_VERSION}/bdist"
DOWNLOAD_URL="https://github.com/coder/code-server/releases/download/v${VSCODE_VERSION}/${TARBALL}"

# Start log
echo "🔧 Installing VSCode Server ${VSCODE_VERSION}"
echo "📁 Installation directory: ${INSTALL_DIR}"

# Create target directory and move into it
mkdir -p "${INSTALL_DIR}"
cd "${INSTALL_DIR}"

# Download the tarball
echo "⬇️  Downloading ${TARBALL} from GitHub..."
wget -q --show-progress "${DOWNLOAD_URL}"

# Extract contents (strip top-level directory)
echo "📦 Extracting ${TARBALL}..."
tar --strip-components=1 -xzf "${TARBALL}"

# Clean up
rm -f "${TARBALL}"

echo "✅ VSCode Server ${VSCODE_VERSION} installed successfully in ${INSTALL_DIR}"
