#!/bin/bash
set -euo pipefail

# Specify VS Code version
VSCODE_VERSION="1.99.3"
INSTALL_DIR="/software/sse2/tetralith_el9/manual/VSCode/${VSCODE_VERSION}/bdist"

echo "🔧 Installing VS Code ${VSCODE_VERSION} into ${INSTALL_DIR}"

# Create and enter the install directory
mkdir -p "${INSTALL_DIR}"
cd "${INSTALL_DIR}"

# Download and extract VS Code
echo "⬇️  Downloading VS Code..."
curl -L -o vscode.tar.gz "https://update.code.visualstudio.com/${VSCODE_VERSION}/linux-x64/stable"

echo "📦 Extracting..."
tar --strip-components=1 -xzf vscode.tar.gz
rm vscode.tar.gz

# Wrap the 'code' binary to always use --no-sandbox
echo "🔧 Patching 'code' launcher to use --no-sandbox (required on Tetralith)"
cd bin
mv code real-code

cat > code <<'EOF'
#!/bin/bash
exec "$(dirname "$0")/real-code" --no-sandbox "$@"
EOF

chmod +x code

echo "✅ VS Code ${VSCODE_VERSION} installed and patched successfully."


# On some HPC clusters or restricted multi-user environments, sandboxing (used by Electron apps like VS Code) can conflict with:

# Kernel security settings (e.g., no user namespaces)
# Restricted file systems (e.g., no /proc write access)
# Older glibc or limited container support

# By using --no-sandbox, you're telling VS Code to skip these security features that are normally required for running in untrusted environments — which makes it work in a shared HPC system.