import os
import sys

def add_conflict_lines(module_names, dry_run=False):
    base_dir_template = "/proj/nsc_testing/xuan/berzelius-module-builds/{module_name}/in-use"

    for module_name in module_names:
        base_dir = base_dir_template.format(module_name=module_name)
        
        # Check if the directory exists
        if not os.path.exists(base_dir):
            print(f"Error: Directory '{base_dir}' does not exist.")
            continue
        
        # Define conflict lines for the module
        conflict_modules = [f'conflict("{other_module}")\n' for other_module in module_names if other_module != module_name]
        
        # Loop over all files in the directory
        for file_name in os.listdir(base_dir):
            if file_name.endswith(".lua"):
                lua_file_path = os.path.join(base_dir, file_name)
                
                try:
                    # Read the existing content of the file
                    with open(lua_file_path, 'r') as file:
                        content = file.readlines()

                    # Check existing conflict lines
                    existing_conflicts = [line for line in content if line.startswith('conflict("Module ')]
                    
                    # Filter out conflict lines that are already present
                    conflicts_to_add = [line for line in conflict_modules if line not in existing_conflicts]

                    if dry_run:
                        if conflicts_to_add:
                            print(f"Dry Run - Conflicts to be added for {lua_file_path}:")
                            for conflict in conflicts_to_add:
                                print(f"  {conflict.strip()}")
                        else:
                            print(f"No conflicts need to be added for {lua_file_path}.")
                    else:
                        if conflicts_to_add:
                            # Add new conflict lines at the beginning of the file
                            new_content = conflicts_to_add + content

                            # Rewrite the file with the new content
                            with open(lua_file_path, 'w') as file:
                                file.writelines(new_content)
                            
                            print(f"Added conflicts to: {lua_file_path}")
                        else:
                            print(f"No conflicts need to be added for {lua_file_path}.")
                        
                except Exception as e:
                    print(f"Error processing {lua_file_path}: {e}")

if __name__ == "__main__":
    if len(sys.argv) < 2:
        print("Usage: python add-conflict-lua.py module_name1 module_name2 ... [--dry-run]")
    else:
        # Check for the dry-run flag
        dry_run = '--dry-run' in sys.argv
        
        # Extract the module names, ignoring the dry-run flag
        module_names = [arg for arg in sys.argv[1:] if arg != '--dry-run']
        
        add_conflict_lines(module_names, dry_run=dry_run)



# python3 add-conflict-lua.py Anaconda Mambaforge Miniforge3 AlphaFold LocalColabFold OpenFold HelixFold3 PyTorch TensorFlow --dry-run