
import os
import sys

def remove_conflict_lines(module_names, dry_run=False):
    base_dir_template = "/proj/nsc_testing/xuan/berzelius-module-builds/{module_name}/in-use"
    
    for module_name in module_names:
        # Set the directory path based on the module_name
        base_dir = base_dir_template.format(module_name=module_name)
        
        # Check if the directory exists
        if not os.path.exists(base_dir):
            print(f"Error: Directory '{base_dir}' does not exist.")
            continue
        
        # Loop over all files in the directory
        for file_name in os.listdir(base_dir):
            if file_name.endswith(".lua"):
                lua_file_path = os.path.join(base_dir, file_name)
                
                try:
                    with open(lua_file_path, 'r') as file:
                        lines = file.readlines()

                    # Filter out the lines that start with "conflict("
                    new_lines = [line for line in lines if not line.strip().startswith("conflict(")]
                    
                    if dry_run:
                        print(f"Dry Run - File: {lua_file_path}")
                        removed_lines = set(lines) - set(new_lines)
                        if removed_lines:
                            print(f"  Lines to be removed: {removed_lines}")
                    else:
                        # Rewrite the file without the conflict lines
                        with open(lua_file_path, 'w') as file:
                            file.writelines(new_lines)
                        print(f"Processed: {lua_file_path}")
                except Exception as e:
                    print(f"Error processing {lua_file_path}: {e}")

if __name__ == "__main__":
    if len(sys.argv) < 2:
        print("Usage: python remove-conflict-lua.py module_name1 module_name2 ... [--dry-run]")
    else:
        # Check for the dry-run flag
        dry_run = '--dry-run' in sys.argv
        
        # Extract the module names, ignoring the dry-run flag
        module_names = [arg for arg in sys.argv[1:] if arg != '--dry-run']
        
        remove_conflict_lines(module_names, dry_run=dry_run)


# python3 remove-conflict-lua.py Anaconda Mambaforge Miniforge3 AlphaFold LocalColabFold OpenFold HelixFold3 PyTorch TensorFlow --dry-run